#!/usr/bin/fades

import os
import shutil
import time
import webbrowser
from urllib import request

import flickrapi  # fades

APIKEY = open("/home/facundo/.flickr/api_key.txt").read().strip()
SECRET = open("/home/facundo/.flickr/sharedsecret.txt").read().strip()
NSID = "54757453@N00"

LOCAL_BASE_DIR = "/home/facundo/Dropbox/.externals/fotos"
assert os.path.exists(LOCAL_BASE_DIR)


def get_all_photos(flickr):
    print("Getting all remote photo list")
    page = 1
    all_photos = []
    while True:
        print("Searching, page", page)
        resp = flickr.photos.search(user_id="me", page=str(page), per_page=500)
        assert resp['stat'] == 'ok'
        photos = resp['photos']
        all_photos.extend(photos['photo'])
        if int(photos['page']) == photos['pages']:
            # we're right in the last page
            break
        page += 1
        time.sleep(.4)  # be gentle
    return all_photos


def download_photo(flickr, photo_id, photo_name):
    resp = flickr.photos.getSizes(photo_id=photo_id)
    for size_data in resp['sizes']['size']:
        if size_data['label'] == 'Original':
            photo_url = size_data['source']
            break
    else:
        print("ERROR!!!!!!! Original size not found for photo id", repr(photo_id))
        return

    # download image to temp file
    http_resp = request.urlopen(photo_url)
    temp_photo_name = photo_name + ".tmp"
    with open(temp_photo_name, "wb") as fh:
        shutil.copyfileobj(http_resp, fh)

    # all fine, rename to proper name
    os.rename(temp_photo_name, photo_name)


def main():
    print("Getting all local photos")
    all_local_names = set()
    for dirpath, dirnames, filenames in os.walk(LOCAL_BASE_DIR):
        all_local_names.update(filenames)
    print("    found {} unique names".format(len(all_local_names)))

    print("Authenticating...")
    flickr = flickrapi.FlickrAPI(APIKEY, SECRET, format='parsed-json')
    if not flickr.token_valid(perms='read'):
        # get a request token
        flickr.get_request_token(oauth_callback='oob')

        # open a browser at the authentication URL. Do this however
        # you want, as long as the user visits that URL.
        authorize_url = flickr.auth_url(perms='read')
        webbrowser.open_new_tab(authorize_url)

        # get the verifier code from the user. Do this however you
        # want, as long as the user gives the application the code.
        verifier = input('Verifier code from the web page just opened: ')

        # trade the request token for an access token
        flickr.get_access_token(verifier)

    all_remote_photos = get_all_photos(flickr)
    print("Found {} remote photos".format(len(all_remote_photos)))

    # ensure we don't have repeated names
    by_name = {}
    for photo in all_remote_photos:
        by_name.setdefault(photo['title'], []).append(photo)
    by_unique_name = {}
    for name, photos in by_name.items():
        if len(photos) == 1:
            by_unique_name[name] = photos[0]
        else:
            for i, photo in enumerate(photos):
                unique_name = "{:010d}-{}".format(i, name)
                by_unique_name[unique_name] = photo

    # find out what's missing
    all_remote_names = set(by_unique_name)
    missing_locally = all_remote_names - all_local_names

    print("Missing locally:", len(missing_locally))
    for idx, name in enumerate(missing_locally):
        if os.path.exists(name):
            continue
        print("    Downloading", idx, name)
        photo = by_unique_name[name]
        download_photo(flickr, photo['id'], name)

    print("Done.")


if __name__ == "__main__":
    main()
